@@ -1,6 +1,5 @@
 import unittest
-
-from mock import Mock, patch
+from unittest.mock import Mock, patch
 
 from patroni.exceptions import PatroniException
 from patroni.utils import Retry, RetryFailedError, enable_keepalive, polling_loop, validate_directory, unquote
